/*************************************************************************************
 **  Q-KernelFree  Copyright (C) 2013 QuasarSoft Ltd.                               **
 **                                                                                 **
 **  This file is part of Q-KernelFree distribution.                                **
 **  Q-KernelFree is free software:  you can redistribute it and/or modify it under **
 **  the terms of the  GNU  General  Public  License  (version 3) as published  by  **
 **  the Free Software  Foundation ***** AND MODIFIED BY ***** the QuasarSoft Ltd.  **
 **  exception.                                                                     **
 **                                                                                 **
 **    *********************** The QuasarSoft Ltd. EXCEPTION ******************     **
 **    * You may not exercise any of the rights granted to You in any manner  *     **
 **    * that is primarily intended for or directed toward commercial         *     **
 **    * advantage or private monetary compensation. The exchange of the      *     **
 **    * Program for other copyrighted works by means of digital file-sharing *     **
 **    * or otherwise shall not be considered to be intended for or directed  *     **
 **    * toward commercial advantage or private monetary compensation,        *     **
 **    * provided there is no payment of any monetary compensation in         *     **
 **    * connection with the exchange of copyrighted works.                   *     **
 **    ************************************************************************     **
 **                                                                                 **
 **  Q-KernelFree is distributed in the hope that it will be useful,  but  WITHOUT  **
 **  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS  ** 
 **  FOR  A  PARTICULAR  PURPOSE.  See the full license text at the following link  ** 
 **  <http://www.quasarsoft.com/license.html>                                       ** 
 **                                                                                 **
 *************************************************************************************/  

#include "xKernel.h"

//
// This function is an alternative for the yRtcc with 0.5 Sec tick because we need a bit 
// finer timing for some sensors. The sensors need an interrupt every 250 mSec
// Timer 1 can provide this.
// 
// The timer uses the 31.25kHz internal oscillator to create a 250mSec interrupt.
// Because 31250/4 is 7812.5 we change the PR value from 7812 to 7813
// From the 250mSec a 1 Minute interrupt is created
//
// If the 250mSec has been expired the function calls the function specified in SetRtccFunction().
//

uint32_t qvUpTime = 0;                  // UpTime of the device
uint32_t qvAlarm = 0;                   // The alarm Date-Time in DTM format (nbr sec since 1/1/2000 00:00:00)
uint32_t qvDatTim = 0;                  // Current DateTime in emulation mode
uint8_t  qvHalfSec = 0;                 // 0=in first half  1=second half

unsigned _SecondDiv4 = 0;               // Number of 250 mSec per second from 0 to 3
unsigned _SecondDiv4Counter = 0;        // Counter of 250 mSec Count from 0 to 239
                                        // This is used for the ANT beacon
static void (*_Function)(void) = NULL;  // Function to call every 250mSec

uint8_t yRtccInit() {
    //Create a 250 mSec timer.
    T1CONbits.T1ECS1 = 1;               // TIECS=2 LPRC oscillator
    T1CONbits.TCS = 1;                  // TCS=1 Extended clock source
    TMR1 = 0;
    PR1 = 7812;                         // First 1/4 of a second
    IFS0bits.T1IF = 0;                  // Clear the interrupt flag
    IEC0bits.T1IE = 1;                  // Enable the interrupt
    T1CONbits.TON = 1;                  // And switch the timer on
    return 4;
}

void yRtccSetDatTim(uint32_t DatTim) {
    qCrtEnter();
    qvDatTim = DatTim;
    qCrtExit();
}

uint32_t yRtccGetDatTim() {
    uint32_t datTim;
    qCrtEnter();
    datTim = qvDatTim;
    qCrtExit();
    return datTim;
}    

uint32_t yRtccGetDatTimRounded() {
    uint32_t datTim;
    qCrtEnter();
    datTim = qvDatTim;
    if (_SecondDiv4 >= 2) datTim++;
    qCrtExit();
    return datTim;
}    

unsigned _yRtccGetHalfSecond() {
    if (_SecondDiv4 >= 2) return 1;
    return 0;
}    

void yRtccSetAlarm(uint32_t alarm){
    qCrtEnter();
    qvAlarm = alarm;
    qCrtExit();
}    

uint32_t yRtccGetUptime() {
    uint32_t upTime;
    qCrtEnter();
    upTime = qvUpTime;
    qCrtExit();
    return upTime;
}    

qISR(_T1Interrupt) {
    IFS0bits.T1IF = 0;                          // clear the interrupt flag
    if (PR1==7812)    {                         // We need 7812.5 ticks which is
        PR1 = 7813;                             // impossible so we split it up like this.
    }
    else {
        PR1 = 7812;
    }
    
    _SecondDiv4++;                              // Increment the 1/4 indicator
    if (_SecondDiv4==4) {                       // Done with 1 second?
        _SecondDiv4 = 0;
        qvUpTime++;
        qvDatTim++;
    }

    _SecondDiv4Counter++;
    if (_SecondDiv4Counter==240) {              // did we reach a minute?
        _SecondDiv4Counter = 0;                 // yes clear the counter
    }

    if (_Function!=NULL) _Function();           // Call function if not Null

    if (qvAlarm!=0) {                           // Has the alarm set
        if (qvDatTim>=qvAlarm) {                // Has the alarm expired?
            qvAlarm = 0;                        // Yes, clear
            qFbrSpawnRtcc();                    // and spawn the Rtcc fiber
        }    
    }
}

// Function can not be called from an interrupt level 2 or higher
// This function take care of the situation when the time is retrieved and
// an interrupt fires.
// Function is used by ANT to provide the number of 1/16 of a second
unsigned GetSecondDiv16() {
    unsigned tmr;
    unsigned secondDiv4Counter;
    
    IEC0bits.T1IE = 0;                          // Disable the interrupt
    tmr = TMR1;                                 // Get the timer and the SecondDiv4 in
    secondDiv4Counter = _SecondDiv4Counter;     // synchronized version
    IEC0bits.T1IE = 1;                          // Enable the interrupt

    // We want to divide it in 8 pieces so we can see if it is in the first part of the quater
    // or the second part of the quarter. 
    if (tmr < 977) {    
        return (secondDiv4Counter<<2);
    }
    if (tmr < 2930) {
        return (secondDiv4Counter<<2)+1;
    }
    if (tmr < 4883) {
        return (secondDiv4Counter<<2)+2;
    }
    if (tmr < (6836)) {
        return (secondDiv4Counter<<2)+3;
    }
    return (secondDiv4Counter<<2)+4;
}

void SetRtccFunction(void (*pFunction)()) {
    _Function = pFunction;
}
